unit Dao360;

interface

uses
    Windows, Classes, SysUtils, Variants, ActiveX, ComObj;

type
    //  ڑEؒfNX
    TDao360 = class
    protected
        FEngine: OleVariant;
        FWorkSpace: OleVariant;
        FDatabase: OleVariant;
        FOpened: Boolean;
        FFileName: String;
        FFilePath: String;
        FPassword: String;
    public
        constructor Create();
        destructor Destroy(); override;
        procedure OpenConnection(const APass, APath: String; var AErr: String); overload;
        procedure OpenConnection(const APassPath: String; var AErr: String); overload;
        procedure CloseConnection();
        function This(): TDao360;
        property Opened: Boolean read FOpened;
        property EG: OleVariant read FEngine;
        property WS: OleVariant read FWorkSpace;
        property DB: OleVariant read FDatabase;
        property FileName: String read FFileName;
        property FilePath: String read FFilePath;
        property Password: String read FPassword;
    end;

const

    //  TableDef.Attributes萔
    dbAttachExclusive	= $00010000;
    dbAttachSavePWD		= $00020000;
    dbSystemObject		= $80000002;
    dbAttachedTable		= $40000000;
    dbAttachedODBC		= $20000000;
    dbHiddenObject		= $00000001;

	//  TableDef.Field.Attributes萔
    dbFixedField		= $00000001;
    dbVariableField		= $00000002;
    dbAutoIncrField		= $00000010;
    dbUpdatableField	= $00000020;
    dbSystemField		= $00002000;
    dbHyperlinkField	= $00008000;
    dbDescending		= $00000001;

	//  Property.Type萔
    dbBoolean	= $00000001;
    dbByte		= $00000002;
    dbInteger	= $00000003;
    dbLong		= $00000004;
    dbCurrency	= $00000005;
    dbSingle	= $00000006;
    dbDouble	= $00000007;
    dbDate		= $00000008;
    dbBinary	= $00000009;
    dbText		= $0000000A;
    dbLongBinary = $0000000B;
    dbMemo		= $0000000C;
    dbGUID		= $0000000F;
    dbBigInt	= $00000010;
    dbVarBinary	= $00000011;
    dbChar		= $00000012;
    dbNumeric	= $00000013;
    dbDecimal	= $00000014;
    dbFloat		= $00000015;
    dbTime		= $00000016;
    dbTimeStamp	= $00000017;

    //  RelationAttributeEnum ̂߂̒萔
    dbRelationUnique        = $00000001;    //  H  QƐH
    dbRelationDontEnforce   = $00000002;    //  H  QƐȂH
    dbRelationInherited     = $00000004;    //  H
    dbRelationUpdateCascade = $00000100;    //  AXV
    dbRelationDeleteCascade = $00001000;    //  A폜
    dbRelationLeft          = $01000000;    //  O
    dbRelationRight         = $02000000;    //  EO

var
    GMdb: TDao360 = nil;

//  t@CpXƃpX[h̋؂蕶
const
    DELIM_PATHPASS: Char = '|';

function Quote(const AText: String): String;
function DoubleQuote(const AText: String): String;

function GetTableAttr(const AAttr: Integer): String;
function GetFieldAttr(const AAttr: Integer): String;
function GetType(AType: Integer): String;
function GetRelationAttr(AAttr: Integer): String;

implementation

//-----------------------------------------------------------------------------
//  UTILITY
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  pǉ
function Quote(const AText: String): String;
begin
    Result := AnsiQuotedStr(AText, '''');
end;

//-----------------------------------------------------------------------------
//  dpǉ
function DoubleQuote(const AText: String): String;
begin
    Result := AnsiQuotedStr(AText, '"');
end;

//-----------------------------------------------------------------------------
//  TDao360
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  RXgN^
constructor TDao360.Create();
begin
    FEngine := CreateOleObject('DAO.DBEngine.36');
    FWorkSpace := FEngine.Workspaces[0];
    FDatabase := Unassigned;
    FOpened := False;
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TDao360.Destroy();
begin
    CloseConnection();
    FWorkSpace := Unassigned;
    FEngine := Unassigned;
end;

//-----------------------------------------------------------------------------
//  ڑ
procedure TDao360.OpenConnection(const APass, APath: String; var AErr: String);
var
    LPass: String;
begin
    //  Ƃ肠
    CloseConnection();

    //  pX[hw蕶
    LPass := Format(';pwd=%s', [APass]);
    try
        //  LEǎpŊJ
        FDatabase := FWorkSpace.OpenDatabase(APath, False, True, LPass);
        //  J
        FOpened := True;

        FFilePath := APath;
        FFileName := ExtractFileName(FFilePath);
        FPassword := APass;
    except
        on E:Exception do
        begin
            AErr := E.Message;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  "pX[h"|"t@CpX" `
procedure TDao360.OpenConnection(const APassPath: String; var AErr: String);
var
    LBreaker: TStringList;
    LPass: String;
    LPath: String;
begin
    LBreaker := nil;
    try
        LBreaker := TStringList.Create();
        try
            LBreaker.Delimiter := DELIM_PATHPASS;
            LBreaker.DelimitedText := APassPath;
            if (LBreaker.Count > 1) then
            begin
                LPass := LBreaker.Strings[0];
                LPath := LBreaker.Strings[1];
            end;
        except
            ;
        end;
    finally
        LBreaker.Free();
    end;

    OpenConnection(LPass, LPath, AErr);
end;

//-----------------------------------------------------------------------------
//  ؒf
procedure TDao360.CloseConnection();
begin
    if FOpened then FDatabase.Close;
    FOpened := False;
    FDatabase := Unassigned;

    FFilePath := '';
    FFileName := '';
    FPassword := '';
end;

//-----------------------------------------------------------------------------
//  ւ̎Q
function TDao360.This(): TDao360;
begin
    Result := Self;
end;

//-----------------------------------------------------------------------------
//  e[ȗ|
function GetTableAttr(const AAttr: Integer): String;
begin
    Result := '';

    if (Aattr and dbAttachExclusive) <> 0 then
        Result := Result + ',rModeŊJLinkTable';

    if (Aattr and dbAttachSavePWD)   <> 0 then
        Result := Result + ',UserIDPasswordڑƋɕۑRemoteTable';

    if (Aattr and dbSystemObject)    <> 0 then
        Result := Result + ',System Table';

    if (Aattr and dbHiddenObject)    <> 0 then
        Result := Result + ',\Table';

    if (Aattr and dbAttachedTable)   <> 0 then
        Result := Result + ',ODBC DataSourceLinkTable';

    if (Aattr and dbAttachedODBC)    <> 0 then
        Result := Result + ',ODBC DataSourceLinkTable';

    Delete(Result, 1, 1);
    if (Result = '') then Result := '@';
end;

//-----------------------------------------------------------------------------
//  tB[h̑|
function GetFieldAttr(const AAttr: Integer): String;
begin
    Result := '';

    if (AAttr and dbFixedField) <> 0 then
        Result := Result + ',SizeŒ';

    if (AAttr and dbVariableField) <> 0 then
        Result := Result + ',Size';

    if (AAttr and dbAutoIncrField) <> 0 then
        Result := Result + ',AutoNumber';

    if (AAttr and dbUpdatableField) <> 0 then
        Result := Result + ',FieldlύX';

    if (AAttr and dbSystemField) <> 0 then
        Result := Result + ',ReplicationL';

    if (AAttr and dbHyperlinkField) <> 0 then
        Result := Result + ',HyperLinkL';

    if (AAttr and dbDescending) <> 0 then
        Result := Result + ',~Sort';

    Delete(Result, 1, 1);
    if (Result = '') then Result := '@';
end;

//-----------------------------------------------------------------------------
//  ^|
function GetType(AType: Integer): String;
begin
	case AType of
		dbBoolean	: Result := 'Bool^(Boolean)';
		dbByte		: Result := 'Byte^(Byte)';
        dbInteger	: Result := '^(Integer)';
        dbLong		: Result := '^(Long)';
        dbCurrency	: Result := 'ʉ݌^(Currency)';
        dbSingle	: Result := 'Px_^(Single)';
        dbDouble	: Result := '{x_^(Double)';
        dbDate		: Result := 't/^(Date/Time)';
        dbBinary	: Result := 'Binary^(Binary)';
        dbText		: Result := 'Text^(Text)';
        //dbLongBinary: Result := 'LongBinary^(LongBinary) OLE Object^(OLE Object)';
        dbLongBinary: Result := 'OLE Object^(OLE Object)';
        dbMemo		: Result := 'Memo^(Memo)';
        dbGUID		: Result := 'GUID^(GUID)';
        dbBigInt	: Result := 'Big Integer^(Big Integer)';
        dbVarBinary	: Result := 'ϒBinary^(VarBinary)';
        dbChar		: Result := 'CHAR^(Char)';
        dbNumeric	: Result := 'Numeric^(Numeric)';
        dbDecimal	: Result := '10i^(Decimal)';
        dbFloat		: Result := '_^(Float)';
        dbTime		: Result := '^(Time)';
        dbTimeStamp	: Result := 'TimeStamp^(TimeStamp)';
    	else		  Result := 'UnKnown (' + IntToStr(AType) + ')'
	end;
end;

//-----------------------------------------------------------------------------
//  [V|
function GetRelationAttr(AAttr: Integer): String;
begin
    Result := '';

    if (AAttr and dbRelationUnique) <> 0 then
        Result := Result + ',RelationUnique';

    if (AAttr and dbRelationDontEnforce) <> 0 then
        Result := Result + ',RelationDontEnforce';

    if (AAttr and dbRelationInherited) <> 0 then
        Result := Result + ',dbRelationInherited';

    if (AAttr and dbRelationUpdateCascade) <> 0 then
        Result := Result + ',FieldAXV';

    if (AAttr and dbRelationDeleteCascade) <> 0 then
        Result := Result + ',RecordA폜';

    if (AAttr and dbRelationLeft) <> 0 then
        Result := Result + ',O';

    if (AAttr and dbRelationRight) <> 0 then
        Result := Result + ',EO';

    Delete(Result, 1, 1);
    if (Result = '') then Result := '@';
end;

//-----------------------------------------------------------------------------
initialization
    CoInitialize(nil);
    GMdb := TDao360.Create();
finalization
    GMdb.Free();
    CoUninitialize();
end.

